import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_error_handler/nui_error_models.dart';

import 'nui_error_handler.dart';

class NUIErrorPopUpDialog extends StatefulWidget {
  NUIErrorPopUpDialog(this.errorMapping, {this.listener});
  final NUIErrorMapping errorMapping;
  final ErrorRedirectCallback listener;
  @override
  _NUIErrorPopUpDialogState createState() => _NUIErrorPopUpDialogState();
}

class _NUIErrorPopUpDialogState extends State<NUIErrorPopUpDialog> {
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.all(25),
      padding: EdgeInsets.only(left: 5, right: 5, top: 5, bottom: 5),
      decoration: BoxDecoration(
          color: NUIColors.NUIWhite,
          borderRadius: BorderRadius.circular(15)
      ),
      child: NUIColumn(
        divider: Divider(height: 5, color: NUIColors.NUITransparent),
        mainAxisSize: MainAxisSize.min,
        children: [
//          BottomSheetBarIndicatorUI(),
          Padding(
            padding: const EdgeInsets.only(top: 10, left: 15, right: 15, bottom: 10),
            child: NUIColumn(
                divider: Divider(height: 5, color: NUIColors.NUITransparent),
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.center,
                mainAxisSize: MainAxisSize.min,
                children: [
                  Text(widget.errorMapping.title, style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: NUIColors.NUITextBlack, decoration: TextDecoration.none)),
                  Text(widget.errorMapping.message, style: TextStyle(fontSize: 14, color: NUIColors.NUITextGray, decoration: TextDecoration.none)),
                  Align(
                    alignment: Alignment.centerRight,
                    child: Visibility(
                      visible: !widget.errorMapping.actionMessage.isNullOrEmpty(),
                      child: NUIRippleContainer(
                        normalColor: NUIColors.NUITransparent,
                        rippleColor: NUIColors.NUIBlack.withOpacity(0.1),
                        borderRadius: BorderRadius.circular(10),
                        child: Padding(
                          padding: const EdgeInsets.only(left: 5, right: 5, top: 3, bottom: 3),
                          child: NUIRow(
                            mainAxisSize: MainAxisSize.min,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Text(widget.errorMapping.actionMessage, style: TextStyle(fontSize: 14, color: NUIColors.NUITextBlack, decoration: TextDecoration.none, fontWeight: FontWeight.bold), maxLines: 1),
                              Icon(Icons.keyboard_arrow_right, color: NUIColors.NUITextBlack, size: 20)
                            ],
                            divider: VerticalDivider(width: 3, color: NUIColors.NUITransparent)
                          ),
                        ),
                        onTap: (){
                          //Redirect or something
                          if(widget.listener != null) widget.listener(context, widget.errorMapping.actionCode, widget.errorMapping);
                          pop(context);
                        },
                      ),
                    ),
                  )
                ]
            ),
          )
        ],
      ),
    );
  }
}


class NUIErrorDetailDialog extends StatefulWidget {
  static bool showing = false;
  NUIErrorDetailDialog(this.errorLog);
  final NUIErrorLog errorLog;
  @override
  _NUIErrorDetailDialogState createState() => _NUIErrorDetailDialogState();
}

class _NUIErrorDetailDialogState extends State<NUIErrorDetailDialog> {
  bool showDetail = false;

  @override
  void initState(){
    super.initState();
    NUIErrorDetailDialog.showing = true;
  }

  @override
  void dispose(){
    super.dispose();
    NUIErrorDetailDialog.showing = false;
  }

  Widget _headerBanner(){
    final shortDetail = "${dateToString(widget.errorLog.dateTime, format: NUIDTUtil.FORMAT_dd_MMM_yyyy_hh_mm_a)}${widget.errorLog.screenCode.isNullOrEmpty()?"" : ", ${widget.errorLog.screenCode}"}";

    return Container(
      decoration: NUIBoxDecoration(
        shadowRadius: 5,
        color: NUIColors.NUIWhite,
      ),
      child: Column(
        children: [
          NUIStatusBar(decoration: BoxDecoration(
            color: NUIColors.NUISkeletalGray
          ),),
          Padding(
            padding: EdgeInsets.only(top: 20, bottom: 20, left: 25, right: 25),
            child: NUIColumn(
              divider: Divider(height: 5, color: NUIColors.NUITransparent),
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(widget.errorLog.title, style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: NUIColors.NUITextBlack, decoration: TextDecoration.none), maxLines: 2, overflow: TextOverflow.ellipsis,),
                Text(shortDetail, style: TextStyle(fontSize: 12, color: NUIColors.NUITextGray, decoration: TextDecoration.none)),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Widget _detailContent(){
    return Expanded(
      child: SingleChildScrollView(
        child: Container(
          color: NUIColors.NUISkeletalLightGray,
          padding: EdgeInsets.all(25),
            child: Text(widget.errorLog.description, style: TextStyle(fontSize: 12, color: NUIColors.NUITextBlack, decoration: TextDecoration.none))
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Column(
        mainAxisSize: MainAxisSize.max,
        children: [
          _headerBanner(),
          _detailContent()
        ],
      ),
    );
  }
}

