import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:excel/excel.dart';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_error_handler/nui_error_handler.dart';
import 'package:nui_error_handler/nui_error_models.dart';

class NUIErrorUtils{
  static const EXCEL_TABLE_NAME = "NUI Error Mapping";
  NUIErrorUtils._builder();

  static List<NUIErrorMapping> readErrorMappingFromJson({@required String value, String json: "defaultvalue"}){
    try {
      final endValue = jsonDecode(json);
      List<NUIErrorMapping> list = (endValue as List).map((i) => NUIErrorMappingEntity().toEntity(i)).toList();
     return list;
    }catch(e){
      return null;
    }
  }

  static List<NUIErrorMapping> readErrorMappingFromExcel(Uint8List bytes) {
    readErrorMappingFromJson();
    try {
      var excel = Excel.decodeBytes(bytes);
      final List<NUIErrorMapping> errorMappings = List();

      for (var table in excel.tables.keys) {
        if(match(table, EXCEL_TABLE_NAME)){
          logNUI(NUIErrorHandler.MODULE, "Found the table specified for error mapping");
          final sheet = excel.tables[table];
          bool skippedHeader = false;
          for (var row in sheet.rows) {
            if(!skippedHeader){
              skippedHeader = true;
              continue;
            }
            final code = getIfAvailable(row, 0)?.toString();
            final module = getIfAvailable(row, 1)?.toString();
            final title = getIfAvailable(row, 2)?.toString();
            final message = getIfAvailable(row, 3)?.toString();
            final actionType = getIfAvailable(row, 4)?.toString();
            final actionCode = getIfAvailable(row, 5)?.toString();
            final actionMessage = getIfAvailable(row, 6)?.toString();
            final extra = getIfAvailable(row, 7)?.toString();

            logNUI("NUIErrorUtils", "code: $code, module : $module, title: $title, message: $message, actionType: $actionType, actionCode: $actionCode, actionMessage: $actionMessage");
            final errorMap = NUIErrorMapping(code: code, module: module, title: title, message: message, actionMessage: actionMessage, actionCode: actionCode, actionType: actionType, extra: extra);
            errorMappings.add(errorMap);
          }
        }
      }
      return errorMappings;
    }catch(e){
      logNUI(NUIErrorHandler.MODULE, "Failed to read error mapping from excel file with error : $e");
      return [];
    }
  }
}